//
//  YHMyDocumentsListViewController.swift
//  galaxy
//
//  Created by EDY on 2024/3/8.
//  Copyright © 2024 https://www.galaxy-immi.com. All rights reserved.
//

import UIKit
/*
 *我的文书 列表
 */

class YHMyDocumentsListViewController: YHBaseViewController {
    var tableView: UITableView!
    var dataSource: [YHMyDocumentsListModel]?
    var viewModel: YHMyDocumentsListViewModel?
    var orderId: Int = 0
    var mobile: String = ""
    var arrIDs : [Int] = []//保存所有下载的id
    
    
    private var isEditFlag : Bool = false
    
    lazy var headSessionView : YHMyDocListHeaderView = {
        let view = YHMyDocListHeaderView()
        return view
    }()
    
    
    lazy var bottomView : YHDocListBottomView = {
        let view = YHDocListBottomView()
        return view
    }()
    
    //头部背景图片
    lazy var headBkgImagV: UIImageView = {
        let imagV = UIImageView()
        imagV.image = UIImage(named: "my_doc_head_bkg")
        imagV.contentMode = .scaleAspectFill
        return imagV
    }()
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        viewModel = YHMyDocumentsListViewModel()
        gk_navTitle = "我的文书"
        setView()
        getData()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        getData()
    }
}

extension YHMyDocumentsListViewController {
    func getData() {
        viewModel?.requestDocumentsList("\(orderId)", mobile, callBackBlock: {[weak self] success, error in
            guard let self = self else { return }
            self.tableView.reloadData()
        })
    }
    
    func updateDataSource() {
       
        self.tableView.reloadData()
    }
    
    func setView() {
        //1
        view.addSubview(headBkgImagV)
        headBkgImagV.snp.makeConstraints { make in
            make.top.equalToSuperview()
            make.left.right.equalToSuperview()
            make.height.equalTo(312)
        }
        
        view.addSubview(headSessionView)
        headSessionView.snp.makeConstraints { make in
            make.top.equalToSuperview().offset(k_Height_NavigationtBarAndStatuBar)
            make.left.equalToSuperview().offset(kMargin)
            make.right.equalToSuperview().offset(-kMargin)
            make.height.equalTo(YHMyDocListHeaderView.viewH)
        }
        
        headSessionView.block = { type in
            self.updateUIForClickButton(type: type)
            self.isEditFlag = type == .downAll ? true : false
            
            guard let model = self.viewModel?.mainModel else { return }
            for item in model.docs {
                item.isEditFlag = self.isEditFlag
                if self.isEditFlag {
                    item.isSelectedFlag = true
                }
            }
            
            self.updateBottomViewNumbers()
            self.tableView.reloadData()
        }
        
        view.addSubview(bottomView)
        bottomView.snp.makeConstraints { make in
            make.bottom.left.right.equalToSuperview()
            make.height.equalTo(YHDocListBottomView.viewH)
        }
        bottomView.isHidden = true
        
        bottomView.block = { (type,flag) in
            if type == .downAllCheck {
                //todo
                self.readyToDownLoad()
            } else if type == .allChoice {
                guard let model = self.viewModel?.mainModel else { return }
                var tag = false
                if flag {
                    //全选
                    tag = true
                } else {
                    //全选 的 反选
                    tag = false
                }
                for item in model.docs {
                    item.isSelectedFlag = tag
                }
                self.updateBottomViewNumbers()
            }
            self.tableView.reloadData()
        }
        
        tableView = {
            let tableView = UITableView(frame:.zero, style:.plain)
            tableView.contentInsetAdjustmentBehavior = .never
            tableView.backgroundColor = .clear
            tableView.separatorStyle = .none
            tableView.delegate = self
            tableView.dataSource = self
            tableView.register(cellWithClass: YHDocListCell.self)
            return tableView
        }()
        
        view.addSubview(tableView)
        tableView.snp.makeConstraints { make in
            make.top.equalTo(headSessionView.snp.bottom)
            make.left.right.equalToSuperview()
            make.bottom.equalToSuperview()
        }
        bottomView.layoutIfNeeded()
        bottomView.bottomButton.addYinHeGradualLayer()
    }
    
    private func updateUIForClickButton(type : ClickButtonType) {
        bottomView.snp.removeConstraints()
        tableView.snp.removeConstraints()
        if type == .downAll {
            bottomView.isHidden = false
            bottomView.snp.remakeConstraints { make in
                make.bottom.left.right.equalToSuperview()
                make.height.equalTo(YHDocListBottomView.viewH)
            }
            
            tableView.snp.remakeConstraints { make in
                make.top.equalTo(headSessionView.snp.bottom)
                make.left.right.equalToSuperview()
                make.bottom.equalTo(bottomView.snp.top)
            }
        } else if type == .cancel {
            bottomView.isHidden = true
            tableView.snp.remakeConstraints { make in
                make.top.equalTo(headSessionView.snp.bottom)
                make.left.right.equalToSuperview()
                make.bottom.equalToSuperview()
            }
        }
    }
    
    private func updateBottomViewNumbers() {
        guard let dataSource = viewModel?.mainModel else { return }
        let count = dataSource.docs.count
        var selectedCount : Int = 0
        for item in dataSource.docs {
            if item.isSelectedFlag {
                selectedCount += 1
            }
        }
        bottomView.updateNumber(allNumber: count, selectedNumber: selectedCount)
    }
    
    private func readyToDownLoad() {
        
        guard let dataSource = viewModel?.mainModel else {
            YHHUD.flash(message: "暂无数据")
            return
        }
        
        arrIDs.removeAll()
        for item in dataSource.docs {
            if item.isSelectedFlag {
                arrIDs.append(item.id)
            }
        }
        
        if arrIDs.isEmpty {
            YHHUD.flash(message: "至少选择一个下载项")
            return
        }
        
        //弹窗 选择发送邮箱
        if let orderID = UserDefaults.standard.value(forKey: "orderIdForPreview") {
            let params = ["order_id":orderID,"ids":arrIDs,"email":"383913637@qq.com"]
            viewModel?.sendEmailForDocs(params, callBackBlock: { success, error in
                if success {
                    YHHUD.flash(message: "发送成功")
                } else {
                    let msg = error?.errorMsg ?? "发送失败"
                    YHHUD.flash(message: msg)
                }
            })
        } else {
            YHHUD.flash(message: "订单ID 为空")
        }
    }
}

extension YHMyDocumentsListViewController: UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return viewModel?.mainModel?.docs.count ?? 0
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withClass: YHDocListCell.self)
        cell.dataSourceForDocs = viewModel?.mainModel?.docs[indexPath.row]
        return cell
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return UITableView.automaticDimension
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        
        guard let  dataSource = viewModel?.mainModel else { return }
        if self.isEditFlag {
            //编辑状态下的点击
            let model = dataSource.docs[indexPath.row]
            model.isSelectedFlag = !model.isSelectedFlag
            tableView.reloadRows(at: [indexPath], with: UITableView.RowAnimation.none)
            updateBottomViewNumbers()
        } else {
            let model = dataSource.docs[indexPath.row]
            let vc = YHMyDocumentsDetailViewController()
            vc.id = model.id
            self.navigationController?.pushViewController(vc)
        }
        
    }
}

